//--------------------------------------------------------------------------------
// ITI
//----------------------------------------------------------------------------------

#define ITI_OFF        0
#define ITI_ON         1
#define ITI_WAIT_IN1   2
#define ITI_WAIT_IN0   3
#define ITI_WAIT_TEMPO 4

class C_iti
{
    public:
        byte aig[6]= {255, 255, 255, 255, 255, 255}; // 255 = not used  (+128=position deviee)
        byte in[6] = {255, 255, 255, 255, 255, 255}; // 255 = not used
        byte u_cf  = 255;  // variable u du commutateur de fermeture (pour annuler un iti), 255 = not used
        byte tp    = 1;    // 1=TP, 0=DA
        byte state = ITI_OFF;
        byte dcpt  = 0;
        byte u     = 0;
        byte lock  = 255;
};
C_iti itis[NB_ITI];

void iti_add(byte num, byte u, byte tp, byte cf, byte a0, byte a1, byte a2, byte a3, byte a4, byte a5, byte i0, byte i1, byte i2,  byte i3, byte i4, byte i5, byte lock, byte num_ind, byte u_ind)
{
    itis[num].u    = u;
    itis[num].tp   = tp;
    itis[num].u_cf = cf;
    itis[num].aig[0] = a0; itis[num].aig[1] = a1; itis[num].aig[2] = a2; itis[num].aig[3] = a3; itis[num].aig[4] = a4; itis[num].aig[5] = a5;
    itis[num].in[0]  = i0; itis[num].in[1]  = i1; itis[num].in[2]  = i2; itis[num].in[3]  = i3; itis[num].in[4]  = i4; itis[num].in[5]  = i5;
    itis[num].lock   = lock;
    sig_iti_add2(num_ind, u_ind, cf, num);
}

//void iti_init(void)
//{
//    // ITI 2  C1->3
//    //      num   u  tp   cf        aig    aig    aig  aig aig aig     in in in in in in
//    iti_add(  2,  2,  0, 255,     1+DIR, 3+DEV, 4+DEV,   X,  X,  X,     4, 5, X, X, X, X);
//
//}

void iti_set(byte num)
{
    byte i, aig, pos;
    C_iti* p_iti;

    p_iti = &itis[num];
    if(p_iti->state != ITI_OFF) return;

    // la ZA doit être libre
    for(i = 0; i < 6; i++)
    {
        if(p_iti->in[i] == 255) break;         // on continue
        if(user_get_in(p_iti->in[i])) return;  // ZA occupé, on retourne
    }

    // verif aiguillages non verouilles
    for(i = 0; i < 6; i++)
    {
        if(p_iti->aig[i] == 255) break;       // on continue
        if(io_aig_lock[p_iti->aig[i] & 0x7f]) return;// aiguillage verouillé, on retourne
    }

    // verif du verrou
    if(p_iti->lock != 255) if(io_lock[p_iti->lock]) return; // verrou, on retourne

    // reservation des aiguillages
    for(i = 0; i < 6; i++)
    {
        if(p_iti->aig[i] == 255) break;
        io_aig_lock[p_iti->aig[i] & 0x7f] = 1; // verouillage aiguillage
    }
    // reservation verrou
    if(p_iti->lock != 255) io_lock[p_iti->lock] = 1;

    // commande des aiguillages
    for(i = 0; i < 6; i++)
    {
        if(p_iti->aig[i] == 255) break;       // on continue
        byte aig = p_iti->aig[i] & 0x7f;
        p_iti->aig[i] & 0x80 ? pos = 1 : pos = 0;
        io_aig_cmd[aig] = pos;  //UU: ?????
        user_notify_aig(aig, pos);
    }

    // declarer l'iti valide
    user_set_u(p_iti->u, 1);
    if(p_iti->tp == 0) p_iti->state = ITI_WAIT_IN1; // DA
    else               p_iti->state = ITI_ON;       // TP
}

void iti_clr(byte num)
{
    byte i;
    C_iti* p_iti;

    p_iti = &itis[num];
    if(p_iti->state == ITI_OFF) return;

    if(p_iti->u_cf != 255) if(user_get_u(p_iti->u_cf) == 0) return;  // if CF exist and OFF

    // here no CF or CF ON to allow destruction
    
    for(i = 0; i < 6; i++)
    {
        if(p_iti->aig[i] == 255) break;
        io_aig_lock[p_iti->aig[i] & 0x7f] = 0; // dereservation des aiguillages
    }
    if(p_iti->lock != 255) io_lock[p_iti->lock] = 0;
    p_iti->state = ITI_OFF;
    user_set_u(p_iti->u, 0);
}

void iti_change(byte num)
{
    user_get_u(itis[num].u) ? iti_clr(num) : iti_set(num);
}
void iti_click(byte num)
{
    iti_change(num);
}

//byte geo_old_u = 255;
//void geo_click(byte u)
//{
//    if(geo_old_u == 255) { geo_old_u = u; user_set_u(u, 1); return; }
//
//    // mettre les correpondances geo -> iti
//    //if(geo_old_u == 70  && u == 73) iti_change(0);
//
//    geo_old_u = 255; user_set_u(geo_old_u, 0);
//
//}

void iti_update(void)
{

    byte num, i, occ;
    C_iti* p_iti;

    for(num = 0; num < NB_ITI; num++)
    {
        yield();

        p_iti = &itis[num];
        if(p_iti->state == ITI_OFF) continue;
        if(p_iti->state == ITI_ON ) continue; // les TP se detruisent à la main

        occ = 0;
        for(i = 0; i < 6; i++)
        {
            if(p_iti->in[i] == 255) break;
            if(user_get_in(p_iti->in[i])) { occ = 1; break; }
        }

        if(p_iti->state == ITI_WAIT_IN1)
        {
            if(occ) p_iti->state = ITI_WAIT_IN0;
        }

        if(p_iti->state == ITI_WAIT_IN0)
        {
            if(occ == 0)
            {
                p_iti->state = ITI_WAIT_TEMPO;
                p_iti->dcpt = 4 * 4;  // 4s = 2*4*250ms
            }
        }

        if(p_iti->state == ITI_WAIT_TEMPO)
        {
            if(occ) { p_iti->state = ITI_WAIT_IN0; continue; }

            if(p_iti->dcpt) p_iti->dcpt--;
            if(p_iti->dcpt == 0)
            {
                for(i = 0; i < 6; i++)
                {
                    if(p_iti->aig[i] == 255) break;
                    io_aig_lock[p_iti->aig[i] & 0x7f] = 0; // dereservation des aiguillages
                }
                if(p_iti->lock != 255) io_lock[p_iti->lock] = 0; // liberation du verrou
                p_iti->state = ITI_OFF;
                user_set_u(p_iti->u, 0);
            }
        }
    }

    sig_iti_update();
}


class C_sig_iti
{
    public:
        byte u = 255;
        byte u_cf = 255;
        byte iti[6] = {255, 255, 255, 255, 255, 255}; // 255 = not used
};
C_sig_iti sig_itis[NB_SIG_ITI];

// maintenant que sig_iti est dans iti_add(), cette fonction n'est plus utilisee
void sig_iti_add(byte num, byte u, byte cf, byte iti0, byte iti1, byte iti2, byte iti3, byte iti4, byte iti5)
{
    sig_itis[num].u    = u;
    sig_itis[num].u_cf = cf;
    sig_itis[num].iti[0] = iti0; sig_itis[num].iti[1] = iti1; sig_itis[num].iti[2] = iti2; sig_itis[num].iti[3] = iti3; sig_itis[num].iti[4] = iti4; sig_itis[num].iti[5] = iti5;
}

void sig_iti_add2(byte num, byte u, byte cf, byte iti)
{
    if(num == 255) return;
    sig_itis[num].u    = u;
    sig_itis[num].u_cf = cf;
    for(byte i = 0; i < 6; i++) if(sig_itis[num].iti[i] == 255) { sig_itis[num].iti[i] = iti; break; }
}

void sig_iti_update(void)
{
    byte num_iti;

    for(byte sig = 0; sig < NB_SIG_ITI; sig++)
    {
        yield();

        byte u = sig_itis[sig].u;
        if(u == 255) continue;

        byte val = 1;
        for(byte i = 0; i < 6; i++)
        {
            num_iti = sig_itis[sig].iti[i];
            if(num_iti == 255) break;
            if((itis[num_iti].state == ITI_ON) || (itis[num_iti].state == ITI_WAIT_IN1))
            {
                val = 0; break;
            }
        }

        byte u_cf = sig_itis[sig].u_cf;
        if(u_cf != 255) if(user_get_u(u_cf)) val = 1;

        user_set_u(u, val);
    }
}
